"""Rename gene tree and species tree based on input species tree using a list of species names"""

import ete3
import numpy as np
from ete3 import Tree, TreeStyle, faces, TextFace, CircleFace, NodeStyle
import pandas as pd
import os
import pdb
import re

gt=None #global tree
gdf=None # global dataframe
gene_id="Rh6"
# Change input and output path name as well as search string in programme
def input_gene_tree():
        "Inputs a gene tree and returns it "
        t=Tree("LEP_LW_opsins_aligned_ML.tre", format=1)
        #t = Tree("((H:0.3,I:0.1):0.5, A:1, (B:0.4,(C:1,(J:1, (F:1, D:1):0.5):0.5):0.5):0.5);")
        return t

def get_leaf_names(t):
        "gets the names of all the leaves and returns a numpy array of the names"
        leaf_labels=np.array([],dtype='object')
        for leaf in t.get_tree_root():
                # Do some analysis on node
                leaf_labels=np.append(leaf_labels,leaf.name)
        return leaf_labels
def input_data_frame():
    "Inputs a data frame needed for analysis"
    path=os.getcwd()
    df=pd.read_csv("../tree_names_all.csv")
    return df

def input_tree():
        "Input species tree"
        t=Tree("../Orig_Fig.S1.tre_1kite-paper.tre")
        return t

def get_leaf_names(t):
        #gets the names of all the leaves and returns a numpy array of the names
        leaf_labels=np.array([],dtype='object')
        for leaf in t.get_tree_root():
                # Do some analysis on node
                leaf_labels=np.append(leaf_labels,leaf.name)
        return leaf_labels

def filter_data(df,crit):
        "Filters an input dataframe using"
        names_list=np.array([""],dtype=object)
        #1. More than 1 opsin
        dh=df["diel-habitat"].astype(str)
        t=input_tree()
        if(crit==1):
                for i in range(dh.size):
                        if(df.number_opsins.iloc[i]>=1):
                                tmp_name=df.genus.iloc[i]+"_"+df.species.iloc[i]
                                df.truename.iloc[i]=search_name_in_tree(tmp_name,t)
                                names_list=np.append(names_list,tmp_name)
        return(names_list,df)

def search_name_in_tree(search_string,t):
        "Input  tree and string to be searched" 
        "Converts the list of leaf names to a list and searches"
        list_of_names=get_leaf_names(t).tolist()
        mylist=list_of_names
        return next((s for s in list_of_names if search_string in s), None)
def find_in_df(search_string):
    "Searches dataframe for partial match and returns index number"
    global gdf
    col=["truename","genus","species","assembly"]
    for i in col:
        x=gdf[gdf[i].str.contains(search_string,na=False)]
        if(not x.empty):
            return x.iloc[0,15]#returns true name
        


def trunc_nam(name):
    "Searches dataframe for partial match and returns index number"
    #Returns the genus and species
    global gdf
    true_names=gdf["truename"].astype(str)
    for i in range(true_names.size):
            if(true_names[i]==name):
                    #ret_val= gdf.superfamily.iloc[i]+" "+gdf.family.iloc[i]+" " + gdf.genus.iloc[i]+" "+gdf.species.iloc[i]
                    ret_val= str(gdf.genus.iloc[i])+"_"+str(gdf.species.iloc[i])
                    
                    break
            else:
                    ret_val=0.0
    return ret_val

def change_leaf_names(t):
        #gets the names of the species tree leaves and changes them based on a lookup function
        leaf_labels=np.array([],dtype='object')
        for leaf in t.get_tree_root():
                # Do some analysis on node
                tmp=leaf.name
                new_name=trunc_nam(tmp)
                if(isinstance(new_name,str)):
                    leaf.name=new_name
        return t
def change_leaf_gene(t,dict_g):
    #Add features to leaves in tree
    for leaf in t.get_tree_root():
        tmp=leaf.name
        species=dict_g.get(tmp)
        global gene_id
        #pdb.set_trace()
        #Adds species names and identifier for gene name to be able to view infig tree
        if(species is not None):
            leaf.name=species + "_"+tmp[-5:]
        else:
            print(leaf.name,"->",species)
                    
    return t
def add_feature_to_leaf(t,dict_g):
    #Add features to leaves in tree
    for leaf in t.get_tree_root():
            leaf.add_features(species=dict_g.get(leaf.name))        
    return t
    return t
def split_returner(s):
    "Takes input string and returns genus,species or identifier"
    
    unit_test=["LEP_SRR2924893_22585_ORF1_Rh6_2",
    "LEP_Myc_nea_C169037_a_79_0_l_1083_ORF1_Rh6",
    "LEP_Ptys_c104231_g1_i1_ORF1_Rh6"]

    x=re.compile('LEP_([A-Z0-9]+?)_').match(s)# Finds SRR29234 like things
    if(x is not None):
        return x.group(1)
    x=re.compile('LEP_([A-Za-z]+?)_([A-Za-z]+?)_').match(s) # Finds matches for genus_species all letters
    if(x is not None):
        output= x.group(1)+"_"+x.group(2)
        return output
    x=re.compile('LEP_([A-Za-z]+?)_').match(s) # Finds and returns Ptys_
    if(x is not None):
        return x.group(1)
def get_dict_leaf_name(t):
    #creates a dictionary of gene tree names and real species names"
    #takes species tree leaf name and returns"
    gene_t=t
    lfn=get_leaf_names(gene_t) #gets leaf names for the gene trees
    sp_identifiers=list(map(split_returner,list(lfn)))# returns identifier to search for in species names
    #Prints tree leaves that have no name
    indexPosList = [ i for i in range(len(sp_identifiers)) if sp_identifiers[i] is None ]
    print("leaves with no matches in the species tree:")
    #following block higlights missing nodes
    for i in indexPosList:
        print(lfn[i])
    #unit test cases
    gene_dict = {}
    for counter,item in enumerate(sp_identifiers):       
        if item is not None:
                sp_in_tree=find_in_df(item)
        else:
            print(item)
        if sp_in_tree is not None :
            gene_dict[lfn[counter]] = trunc_nam(sp_in_tree)
            #print(item,"->",sp_in_tree,"->",trunc_nam(sp_in_tree))
    return gene_dict      


def process():
    #like main
    t=input_tree()
    df=input_data_frame()
    #filtered_names,df=filter_data(df,1)
    #true_names=[""]
    #for s in filtered_names:
    #    true_names.append(search_name_in_tree(s,t))
    renamed_tree=input_tree()

    #true_names=list(filter(None,true_names))
    #rename species tree to genus and species
        
    global gt 
    gt = t
    global gdf 
    gdf = df

    renamed_tree=change_leaf_names(renamed_tree)
    #uncomment line to output species tree
    renamed_tree.write(format=1, outfile="out1_sp_renamed_sp.tree.tre")
    #process gene tree and rename
    gene_t=input_gene_tree()
    gene_to_sp_dict=get_dict_leaf_name(gene_t) # creates a dictionary for branch tree names to genus_species
    gene_t_wn=change_leaf_gene(gene_t,gene_to_sp_dict) 
    #for writing feature name
    #gene_t_wn.write(format=1,features=["species"],outfile="leave_namesdLW_gene.tre")
    gene_t_wn.write(format=1,outfile="out_gene_leave_names_LW_gene.tre")
    
process()