"""
Takes a nexus tree of insects and an csv file from TableS@_insect_opsins after you convert the excel file to a csv.
Prune tree to Diurnal/ nocturnal species et. al""" 

import ete3
import numpy as np
from ete3 import Tree, TreeStyle, faces, TextFace, CircleFace, NodeStyle
import pandas as pd
import os
import sys
count_col = 0
gdf = None
gt = None
def input_tree():
        t=Tree("/Users/apple/Desktop/vis-trans/SMBE/Fig1_t/NCBI_tree_graft_misof_tree.tre", format=1)
        #t = Tree("((H:0.3,I:0.1):0.5, A:1, (B:0.4,(C:1,(J:1, (F:1, D:1):0.5):0.5):0.5):0.5);")
        return t

def input_data_frame():
    #Inputs a data frame needed for analysis
    path=os.getcwd()
    df=pd.read_csv("/Users/apple/Desktop/vis-trans/SMBE/TableS2/cleaned_table.csv")
    return df
def cleanup(df):
    #cleans spaces and makes all column names lowercase
    df=df.rename(columns=lambda x: x.lower().replace(" ", "_"))
    return df
def get_leaf_names(t):
        #gets the names of all the leaves and returns a numpy array of the names
        leaf_labels=np.array([],dtype='object')
        for leaf in t.get_tree_root():
                # Do some analysis on node
                leaf_labels=np.append(leaf_labels,leaf.name)
        return leaf_labels

def search_name_in_tree(search_string,t):
        #Input  tree and string to be searched 
        #converts the list of leaf names to a list and searches 
        list_of_names=get_leaf_names(t).tolist()
        mylist=list_of_names
        return next((s for s in list_of_names if search_string in s), None)
def data_proc(df):
        "takes an input dataframe and cleans based on a criteria and keeps number of rows"
        remove_uncertain=False
        filter_criteria="N and D"                     
        #trim to needed columns
        df=df.iloc[:,[0,1,2,3,4,5,6,7,8,9,10,11]]
        #Rename a specific column to something else
        df=df.rename(columns={df.columns[2]: "diel-habitat"})
        #replace empty characters with 0 
        df=df.fillna(0)
        #dh=Column that contains all the diel habitat states
        if remove_uncertain:
            dh=df["diel-habitat"].astype(str)
        elif not remove_uncertain:
            #Keeps uncertain values of diel states and removes the ? symbol
            df=df.replace({"\?":""},regex=True)    
            dh=df["diel-habitat"].astype(str)
        #Replace ? with empty characters or 0 depending on state
        df=df.replace({"\?":""},regex=True)    
        #replace cleaned columns as float to allow for further processing
        df.rh5_blue=df.rh5_blue.astype(float)                                                                                                  
        df.rh6_lw=df.rh6_lw.astype(float)
        # Adds column for number of opsins
        df.loc[(df.rh4_uv >= 1) | (df.rh5_blue >= 1) | (df.rh6_lw >=1) | (df.rh7 >= 1), "number_opsins" ] = (df.rh4_uv+df.rh5_blue+df.rh6_lw+df.rh7)
        # Adds column for number of duplications
        df.loc[(df.rh4_uv >= 2) | (df.rh5_blue >= 2) | (df.rh6_lw >=2) | (df.rh7 >= 2), "duplications" ] = 1 
        #Adds genus
        return df
def genus_creator(genus_species):
        #Splits genus from genus_species
        temp=genus_species.split("_")[0]
        
        return temp
def species_creator(genus_species):
        #Splits species from genus_species
        temp=genus_species.split(" ")[1]
        return temp

def add_col(df):
        #Adds columns based on lambda functions

        df["genus"]=df.apply(lambda x: genus_creator(x["genus_species"]),axis =1)
        #To use the lambda function choose a function to apply and send the columns to the function using x. axis =1 applies it row wise"
        #Adds species
        df["species"]=df.apply(lambda x: species_creator(x["genus_species"]),axis =1)
        
        df["truename"]=df.apply(lambda x: x["genus_species"].split(" ")[0]+" "+x["genus_species"].split(" ")[1],axis =1)
        return df
def filter_data(df,crit):
        names_list=np.array([""],dtype=object)
        #1. More than 1 opsin
        dh=df["diel-habitat"].astype(str)
        t=input_tree()
        if(crit==1):
                for i in range(dh.size):
                        if(df.number_opsins.iloc[i]>=1):
                                tmp_name=df.genus.iloc[i]+"_"+df.species.iloc[i]
                                df.truename.iloc[i]=search_name_in_tree(tmp_name,t)
                                names_list=np.append(names_list,tmp_name)
        return(names_list,df)
def uv_num(name):
        #returns number of uv opsins
        global gdf
        true_names=gdf["truename"].astype(str)
        for i in range(true_names.size):
                if(true_names[i]==name):
                        ret_val=gdf.rh4_uv.iloc[i]
                        break
                else:
                        ret_val=0.0
        return ret_val
def blue_num(name):
        #returns number of blue opsins
        global gdf
        true_names=gdf["truename"].astype(str)
        for i in range(true_names.size):
                if(true_names[i]==name):
                        ret_val=gdf.rh5_blue.iloc[i]
                        break
                else:
                        ret_val=0.0
        return ret_val
def lw_num(name):
        #returns number of lw opsins
        global gdf
        true_names=gdf["truename"].astype(str)
        for i in range(true_names.size):
                if(true_names[i]==name):
                        ret_val=gdf.rh6_lw.iloc[i]
                        break
                else:
                        ret_val=0.0
        return ret_val    
def rh7_num(name):
        #returns number of rh7 opsins
        global gdf
        true_names=gdf["truename"].astype(str)
        for i in range(true_names.size):
                if(true_names[i]==name):
                        ret_val=gdf.rh7.iloc[i]
                        break
                else:
                        ret_val=0.0
        return ret_val         
def trunc_nam(name):
        #Returns the genus and species
        global gdf
        true_names=gdf["truename"].astype(str)
        for i in range(true_names.size):
                if(true_names[i]==name):
                        ret_val=gdf.genus.iloc[i]+" "+gdf.species.iloc[i]
                        break
                else:
                        ret_val=0.0
        return ret_val
def diel_habit(name):
        #Returns the genus and species
        global gdf
        true_names=gdf["truename"].astype(str)
        for i in range(true_names.size):
                if(true_names[i]==name):
                        ret_val= gdf["diel-habitat"].iloc[i]
                        break
                else:
                        ret_val=0.0
        return ret_val


def process():
        t=input_tree()
        df=input_data_frame()
        #df=cleanup(df)
        #df=data_proc(df)
        #df=add_col(df)
        #filterd_names,df=filter_data(df,1)
        #true_names=[""]
        #for s in filterd_names:
        #       true_names.append(search_name_in_tree(s,t))
        #pruned_tree=input_tree()
        #true_names=list(filter(None,true_names))
        global gt 
        gt = t
        global gdf 
        gdf = df
        gdf.to_csv("/Users/apple/Desktop/vis-trans/SMBE/Fig1_t/cleaned_table.csv")
        
        pruned_tree=t
        #pruned_tree.prune(true_names)
        #pruned_tree.ladderize()
        pruned_tree.convert_to_ultrametric()
        add_faces(pruned_tree)
        #pruned_tree.show()
        #pruned_tree.write(format=1, outfile="insect_graft_opsin_tree.tre")

def add_faces(t):
        # Basic tree style
        ts = TreeStyle()
        ts.force_topology= False
        ts.mode = "r"
        ts.optimal_scale_level="mid"
        ts.layout_fn=layout
        ts.show_leaf_name = False
        t.show(tree_style=ts)
        t.render("mytree.svg",tree_style=ts)
        
def layout(node):
        if node.is_leaf():
                #File format has single quotes around file names which need to be removed
                leaf_name=node.name.replace("'","")
                dh=diel_habit(leaf_name)
                node.img_style["size"]=0
                
                
                
                uv_size=int(uv_num(leaf_name))
                blue_size=int(blue_num(leaf_name))
                lw_size=int(lw_num(leaf_name))*1
                rh7_size=int(rh7_num(leaf_name))*1
                genus_species=trunc_nam(leaf_name)
                
                if(len(genus_species)<28):
                        num_space=(28-len(genus_species))*" "
                gs= TextFace(genus_species+num_space,fstyle="italic")
                shift = 1
                draw_aligned_faces_as_table=True
                
                if(uv_size >1 or blue_size >1 or lw_size >1):
                        node.img_style["size"]=5
                        node.img_style["fgcolor"]="red"
                       
                allow_face_overlap=True
                # changes genus and species color based in deil activit
                if(dh=="Diurnal"):
                        gs.background.color = "gold"
                elif(dh=="Both" or dh =="Crepuscular"):
                        gs.background.color = "grey"
                elif(dh=="Nocturnal"or dh=="Both" or dh =="Crepuscular"):
                        gs.background.color = "aqua"
                faces.add_face_to_node(gs,node, column=0)
                #uv
                uv_face= TextFace("  " +str(uv_size)+"  ")
                uv_face.background.color = "Plum"
                uv_face.border.width = 1
                if(uv_size>1):
                        uv_face.opacity = 1*0.5*uv_size
                        uv_face.background.color = "Orchid"
                elif(uv_size==0):
                        uv_face.opacity = 0.3
                faces.add_face_to_node(uv_face, node, column=2, aligned=True,position = "branch-right")
                
                #blue
                blue_face =TextFace("   " + str(blue_size)+ "  ",fgcolor="black")
                blue_face.background.color ="LightSkyBlue"
                blue_face.border.width=1
                
                if(blue_size>1):
                        blue_face.opacity = 1*0.5*blue_size
                        blue_face.background.color ="SteelBlue"
                elif(blue_size==0):
                        blue_face.opacity = 0.3
                faces.add_face_to_node(blue_face, node, column=3,aligned=True)
                #lw
                lw_face=TextFace("   "+ str(lw_size)+"   ")
                lw_face.background.color ="PaleGreen"
                lw_face.border.width=1
                if(lw_size>1):
                        lw_face.opacity = 1*0.5*lw_size
                        lw_face.background.color ="LimeGreen"
                elif(lw_size==0):
                        lw_face.opacity = 0.3
                faces.add_face_to_node(lw_face, node, column=4,aligned=True)
                #rh7
                rh7_face=TextFace("   "+ str(rh7_size)+"   ")
                rh7_face.background.color ="LightCoral"
                rh7_face.border.width=1
                if(rh7_size>1):
                        rh7_face.opacity = 1*0.5*rh7_size
                        rh7_face.background.color ="Crimson"
                elif(rh7_size==0):
                        rh7_face.opacity = 0.3
                faces.add_face_to_node(rh7_face, node, column=5,aligned=True)
                

                #faces.add_face_to_node(TextFace(str(rh7_size)+" "), node, column=5)
                #black is placeholder to be removed in illustrator
                #global count_col
                #if(count_col==0):      
                #        faces.add_face_to_node(CircleFace(radius=6,color="Black",style="circle"),node, column =shift+11+0,position = "aligned")
                #        count_col+=1
                #
                #faces.add_face_to_node(TextFace("            "),node, column=1) #adds spaces to the names to increase length
        else:
                node.img_style["fgcolor"] = "black"
                node.img_style["size"] = 0
process()