import numpy as np
import matplotlib.pyplot as plt
import pandas as pd
import os
import sys
#Dataframe to be plotted
gdf = None
#Criteria for a-d figure to be plotted see function filter_data for details
crit = None

def input_data_frame():
    #Inputs a data frame needed for analysis
    path=os.getcwd()
    df=pd.read_csv("/Users/apple/Desktop/vis-trans/SMBE/TableS4/TableS4.csv")
    return df
def cleanup(df):
    #cleans spaces and makes all column names lowercase
    df=df.rename(columns=lambda x: x.lower().replace(" ", "_"))
    return df    
# data to plot

def filter_data():
    #Returns filtered dataset
    global gdf
    global crit
    fdf = None
    #Filters the dataset using the loc command. 
    if(crit=="a"):
        filter1=gdf["number_of__taxa"]>=20
        filter2=gdf["tree"] == "gene"
        filter3=gdf["null_omega"] <=0.1
        fdf = gdf.loc[filter1 & filter2 & filter3]
    elif(crit=="b"):
        filter1=gdf["number_of__taxa"]<=20
        filter2=gdf["tree"] == "gene"
        filter3=gdf["number_of__taxa"] >=12
        fdf = gdf.loc[filter1 & filter2 & filter3]
    elif(crit=="c"):
        filter1=gdf["number_of__taxa"]>=20
        filter2=gdf["tree"] == "Species"
        filter3=gdf["number_of__taxa"] >=12
        fdf = gdf.loc[filter1 & filter2 & filter3]

    return fdf

def extract_data():
    #extracts and returns the nocturnal and diurnal w w values from the global data filtered data
    global gdf
    fdf =filter_data()
    nocturnal =fdf.nocturnal_w
    diurnal= fdf.diurnal_w
    return nocturnal,diurnal
def create_plot_3a():
    crit="a"
    n_groups = 4
    fig, ax = plt.subplots()
    index = np.arange(n_groups)
    bar_width = 0.35
    opacity = 0.8
    nocturnal,diurnal = extract_data()
    rects1 = plt.bar(index, nocturnal, bar_width,
    alpha=opacity,
    color='g',
    label='Nocturnal')
    rects2 = plt.bar(index + bar_width, diurnal, bar_width,
    alpha=opacity,
    color='b',
    label='Diurnal')
    index_tot= np.concatenate((index,index+bar_width))
    height_tot = np.concatenate((nocturnal,diurnal))
    for i in range(n_groups):
        #cleans p-value if it has < sign and converts to float
        pvalue=gdf.iloc[i,6]
        pvalue=clean_pvalue(pvalue)
        barplot_annotate_brackets(i,i+4,pvalue,index_tot,height_tot)
    
    plt.xlabel('Opsins')
    plt.ylabel('Omega dN/dS')
    plt.title('dN/dS rates')
    plt.xticks(index + bar_width, ('Rh4 UV', 'Rh5 Blue', 'Rh6 LW', 'Rh7'))
    plt.legend()
    plt.tight_layout()
    #plt.show()
    plt.savefig("Fig3a.svg")
def create_plot_3b():
    global crit
    crit="b"
    n_groups = 3
    
    fig, ax = plt.subplots()
    index = np.arange(n_groups)
    bar_width = 0.35
    opacity = 0.8
    nocturnal,diurnal = extract_data()
    rects1 = plt.bar(index, nocturnal, bar_width,
    alpha=opacity,
    color='g',
    label='Nocturnal')
    rects2 = plt.bar(index + bar_width, diurnal, bar_width,
    alpha=opacity,
    color='b',
    label='Diurnal')
    
    index_tot= np.concatenate((index,index+bar_width))
    height_tot = np.concatenate((nocturnal,diurnal))
    for i in range(n_groups):
        #cleans p-value if it has < sign and converts to float
        pvalue=gdf.iloc[i,6]
        pvalue=clean_pvalue(pvalue)
        barplot_annotate_brackets(i,i+n_groups,pvalue,index_tot,height_tot)
    plt.xlabel('Opsins')
    plt.ylabel('Omega dN/dS')
    plt.title('dN/dS rates')
    plt.xticks(index + bar_width, ('Rh4 UV', 'Rh5 Blue', 'Rh6 LW'))
    plt.legend()
    plt.tight_layout()
    #plt.show()
    plt.savefig("Fig3b.svg")
def create_plot_3c():
    global crit
    crit="c"
    n_groups = 3
    
    fig, ax = plt.subplots()
    index = np.arange(n_groups)
    bar_width = 0.35
    opacity = 0.8
    nocturnal,diurnal = extract_data()
    rects1 = plt.bar(index, nocturnal, bar_width,
    alpha=opacity,
    color='g',
    label='Nocturnal')
    rects2 = plt.bar(index + bar_width, diurnal, bar_width,
    alpha=opacity,
    color='b',
    label='Diurnal')
    
    index_tot= np.concatenate((index,index+bar_width))
    height_tot = np.concatenate((nocturnal,diurnal))
    for i in range(n_groups):
        #cleans p-value if it has < sign and converts to float
        pvalue=gdf.iloc[i,6]
        pvalue=clean_pvalue(pvalue)
        barplot_annotate_brackets(i,i+n_groups,pvalue,index_tot,height_tot)
    plt.xlabel('Opsins')
    plt.ylabel('Omega dN/dS')
    plt.title('dN/dS rates')
    plt.xticks(index + bar_width, ('Rh4 UV', 'Rh5 Blue', 'Rh6 LW'))
    plt.legend()
    plt.tight_layout()
    #plt.show()
    plt.savefig("Fig3c.svg") 
def barplot_annotate_brackets(num1, num2, data, center, height, yerr=None, dh=.05, barh=.05, fs=None, maxasterix=None):
    """ 
    Annotate barplot with p-values.

    :param num1: number of left bar to put bracket over
    :param num2: number of right bar to put bracket over
    :param data: string to write or number for generating asterixes
    :param center: centers of all bars (like plt.bar() input)
    :param height: heights of all bars (like plt.bar() input)
    :param yerr: yerrs of all bars (like plt.bar() input)
    :param dh: height offset over bar / bar + yerr in axes coordinates (0 to 1)
    :param barh: bar height in axes coordinates (0 to 1)
    :param fs: font size
    :param maxasterix: maximum number of asterixes to write (for very small p-values)
    https://stackoverflow.com/questions/11517986/indicating-the-statistically-significant-difference-in-bar-graph
    """

    if type(data) is str:
        text = data
    else:
        # * is p < 0.05
        # ** is p < 0.005
        # *** is p < 0.0005
        # etc.
        text = ''
        p = .05

        while data < p:
            text += '*'
            p /= 10.

            if maxasterix and len(text) == maxasterix:
                break

        if len(text) == 0:
            text = 'n. s.'

    lx, ly = center[num1], height[num1]
    rx, ry = center[num2], height[num2]

    if yerr:
        ly += yerr[num1]
        ry += yerr[num2]

    ax_y0, ax_y1 = plt.gca().get_ylim()
    dh *= (ax_y1 - ax_y0)
    barh *= (ax_y1 - ax_y0)

    y = max(ly, ry) + dh

    barx = [lx, lx, rx, rx]
    bary = [y, y+barh, y+barh, y]
    mid = ((lx+rx)/2, y+barh)

    plt.plot(barx, bary, c='black')

    kwargs = dict(ha='center', va='bottom')
    if fs is not None:
        kwargs['fontsize'] = fs

    plt.text(*mid, text, **kwargs)
def clean_pvalue(pvalue):
    """inputs a p-value and returns the value in a float and removes < if it exists)"""
    if(type(pvalue) == str):
        return float(pvalue.split("<")[1] if "<" in pvalue else pvalue)
    else:
        return float(pvalue)

def run_program():
    df =input_data_frame()
    df = cleanup(df)
    global gdf
    gdf = df
    #create_plot_3a()
    #create_plot_3b()
    create_plot_3c()


run_program()
#plt.savefig()
