"""
Filters opsin mRnA""" 
import numpy as np
from ete3 import Tree
import pandas as pd
import os
import sys
gdf = None
gt = None
df_op = None

def input_tree():
        #Inputs tree
        t=Tree("/Users/apple/Desktop/vis-trans/SMBE/Fig1/Orig_Fig.S1.tre_1kite-paper.tre")
        #Orignal tree from Kawahara paper
        
        return t

def input_data_frame():
    #Inputs a data frame needed for analysis
    path=os.getcwd()
    df=pd.read_csv("/Users/apple/Desktop/vis-trans/SMBE/TableS1/diel-opsin-raw.csv")
    return df
def input_opsin_seq():
        # Inputs opsins as a csv file, from a file exported in genious with all opsins
        df= pd.read_csv("/Users/apple/Desktop/vis-trans/SMBE/Fig3/lw/1.raw_data/all_lep_opsin.csv")
        return df
def cleanup(df):
    #cleans spaces and makes all column names lowercase
    df=df.rename(columns=lambda x: x.lower().replace(" ", "_"))
    return df
def get_leaf_names(t):
        #gets the names of all the leaves and returns a numpy array of the names
        leaf_labels=np.array([],dtype='object')
        for leaf in t.get_tree_root():
                # Do some analysis on node
                leaf_labels=np.append(leaf_labels,leaf.name)
        return leaf_labels

def search_name_in_tree(search_string,t):
        #Input  tree and string to be searched 
        #converts the list of leaf names to a list and searches 
        list_of_names=get_leaf_names(t).tolist()
        
        return next((s for s in list_of_names if search_string in s), None)
def search_queryinlist(search_string, list2):
        #Searches for a query with a search string input and list to be searched in 
        mylist=list2.tolist()
        return next((s for s in mylist if search_string in s), None)
def data_proc(df):
        #Processes and returns a dataframe input data and based on several critera
        remove_uncertain=False
        filter_criteria="N and D"                     
        "takes an input dataframe and cleans based on a criteria"
        #trim to needed columns
        df=df.iloc[:,[0,1,2,3,4,5,15,17,20,21,22,23,24]]
        #Rename a specific column to something else
        df=df.rename(columns={df.columns[7]: "diel-habitat"})
        #replace empty characters with 0 
        df=df.fillna(0)
        #dh=Column that contains all the diel habitat states
        if remove_uncertain:
            dh=df["diel-habitat"].astype(str)
        elif not remove_uncertain:
            #Keeps uncertain values of diel states and removes the ? symbol
            df=df.replace({"\?":""},regex=True)    
            dh=df["diel-habitat"].astype(str)
        #Replace ? with empty characters or 0 depending on stte
        df=df.replace({"\?":""},regex=True)    
        #replace cleaned columns as float to allow for further processing
        df.rh5_blue=df.rh5_blue.astype(float)                                                                                                  
        df.rh6_lw=df.rh6_lw.astype(float)
        # Adds column for number of opsins
        df.loc[(df.rh4_uv >= 1) | (df.rh5_blue >= 1) | (df.rh6_lw >=1) | (df.rh7 >= 1), "number_opsins" ] = (df.rh4_uv+df.rh5_blue+df.rh6_lw+df.rh7)
        # Adds column for number of duplications
        df.loc[(df.rh4_uv >= 2) | (df.rh5_blue >= 2) | (df.rh6_lw >=2) | (df.rh7 >= 2), "duplications" ] = 1 
        df.loc[df.number_opsins >=1,"treename"] = ""
        return df

def filter_data(df,crit):
        #Filters data to criteria  needed
        names_list=np.array([""],dtype=object)
        #1. More than 1 opsin
        #2. Atleast 1 uv, blue and green opsin
        dh=df["diel-habitat"].astype(str)
        t=input_tree()
        if(crit==1):
                for i in range(dh.size):
                        if(df.number_opsins.iloc[i]>=1):
                                tmp_name=df.genus.iloc[i]+"_"+df.species.iloc[i]
                                df.treename.iloc[i]=search_name_in_tree(tmp_name,t)
                                names_list=np.append(names_list,tmp_name)
        elif(crit==2):
                for i in range(dh.size):
                        if((df.rh4_uv.iloc[i]>= 1) and (df.rh5_blue.iloc[i] >= 1) and (df.rh6_lw.iloc[i] >=1)):
                                tmp_name=df.genus.iloc[i]+"_"+df.species.iloc[i]
                                df.treename.iloc[i]=search_name_in_tree(tmp_name,t)
                                names_list=np.append(names_list,tmp_name)

        return(names_list,df)
def lw_seq(tree_name):
        #returns sequence of lw opsins number of lw opsins based on tree_name
        global df_op
        count_opsin=0
        seq_name=df_op["name"].astype(str)
        for i in range(seq_name.size):
                if(df_op.treename.iloc[i]==tree_name and (df_op.name.iloc[i].find("lw")!=-1 or df_op.name.iloc[i].find("rh6")!=-1) ):
                        ret_val=df_op.sequence.iloc[i]
                        count_opsin=count_opsin+1
                        break
                else:
                        ret_val=0.0
        return ret_val,count_opsin
def uv_num(name):
        #returns number of uv opsins
        global gdf
        true_names=gdf["treename"].astype(str)
        for i in range(true_names.size):
                if(true_names[i]==name):
                        ret_val=gdf.rh4_uv.iloc[i]
                        break
                else:
                        ret_val=0.0
        return ret_val
def blue_num(name):
        #returns number of blue opsins
        global gdf
        true_names=gdf["treename"].astype(str)
        for i in range(true_names.size):
                if(true_names[i]==name):
                        ret_val=gdf.rh5_blue.iloc[i]
                        break
                else:
                        ret_val=0.0
        return ret_val
def lw_num(name):
        #returns number of lw opsins
        global gdf
        true_names=gdf["treename"].astype(str)
        for i in range(true_names.size):
                if(true_names[i]==name):
                        ret_val=gdf.rh6_lw.iloc[i]
                        break
                else:
                        ret_val=0.0
        return ret_val    
def rh7_num(name):
        #returns number of rh7 opsins
        global gdf
        true_names=gdf["treename"].astype(str)
        for i in range(true_names.size):
                if(true_names[i]==name):
                        ret_val=gdf.rh7.iloc[i]
                        break
                else:
                        ret_val=0.0
        return ret_val         
def trunc_nam(name):
        #Returns the genus and species
        global gdf
        true_names=gdf["treename"].astype(str)
        for i in range(true_names.size):
                if(true_names[i]==name):
                        ret_val= gdf.genus.iloc[i]+"_"+gdf.species.iloc[i]
                        break
                else:
                        ret_val=0.0
        return ret_val
def diel_habit(name):
        #Returns the genus and species
        global gdf
        true_names=gdf["treename"].astype(str)
        for i in range(true_names.size):
                if(true_names[i]==name):
                        ret_val= gdf["diel-habitat"].iloc[i]
                        break
                else:
                        ret_val=0.0
        return ret_val
def write_fasta(seq_id,seq,file):
        
        #adds a id, seq to a fasta file
        filename=file
        myfile = open(filename, "a+")
        with myfile:
                myfile.writelines(">"+seq_id+"\n")
                myfile.writelines(seq+"\n")

def process():
        #final_soap_FG120049B.fa
        #CLEAN_Dara.fa
        #"library-id_/_id"
        #"assembly"
        t=input_tree()
        #Clean the names db
        df=input_data_frame()
        df=cleanup(df)
        df=data_proc(df)
        #Clean opsin database
        op_df=input_opsin_seq()
        op_df=cleanup(op_df)

        #makes column name all lower case

        #Make all library id lower case
        df["library-id_/_id"]=df["library-id_/_id"].astype(str).str.lower()
        #Make assembly library id lower cas
        df["assembly"]=df["assembly"].astype(str).str.lower()
        op_df.name=op_df.name.astype(str).str.lower()
        seq_names=op_df.name #saves sequence names
        #Filters names and database to just the filter critera
        filtered_names,df=filter_data(df,2)

        # Searches for names in original tree
        tree_names=[""]
        for s in filtered_names:
                tree_names.append(search_name_in_tree(s,t))
        
        #Inputs original tree for pruning
        pruned_tree=input_tree()
        #Gets all names in tree excluding ones with None
        tree_names=list(filter(None,tree_names))
        
        global gt 
        #Saves tree as input tree
        gt = t
        # Saves new database in gdf
        global gdf 
        gdf = df
        #gdf.to_csv("/Users/apple/Desktop/vis-trans/SMBE/TableS1/true-names.csv")
        gdf.to_csv("/Users/apple/Desktop/vis-trans/SMBE/Fig3/lw/1.raw_data/modified_TableS1.csv")
        # Search for all names in s
        op_df.loc[op_df["molecule_type"]=="DNA","treename"] = ""
        for i in range(seq_names.size):
                x=seq_names[i]
                lib=df["library-id_/_id"].astype(str)
                idx_lib=pd.Index(lib)
                assem=df["assembly"].astype(str)
                idx_assem=pd.Index(assem)
                genus=df["genus"].astype(str).str.lower()
                species=df["species"].astype(str).str.lower()
                idx_genus=pd.Index(genus)
                s=x.find("_")
                s_1=x.find("_",s+1)
                s_2=x.find("_",s_1+1)
                tree_name=""
                n=x[s+1:x.find("_",s_2)]
                # N is from first underscore to third underscore

                as_search=search_queryinlist(n,assem)
                if(as_search!=None):
                        op_df.treename.iloc[i]=gdf.treename[idx_assem.get_loc(as_search)]
                        continue
                elif(as_search==None):
                        n=x[s+1:s_1]
                        lib_search=search_queryinlist(n,lib)
                        if(lib_search!=None):
                                op_df.treename.iloc[i]=gdf.treename[idx_lib.get_loc(lib_search)]
                                continue
                        elif(lib_search==None):
                                genus_search=search_queryinlist(n,genus)
                                if(genus_search==None):
                                        print(x,"=",n)
                                else:
                                        op_df.treename.iloc[i]=gdf.treename[idx_genus.get_loc(genus_search)]
        global df_op
        df_op= op_df                 
        op_df.to_csv("/Users/apple/Desktop/vis-trans/SMBE/Fig3/lw/1.raw_data/opsin_with_treenames.csv") 
        #Does not have all true names, but most, the few that need to be added to true names are in the word file                                          
        pruned_tree.prune(tree_names)

        filename="lw_dna.fasta"
        f=open(filename,"w+")
        f.close()

        for i in tree_names:  
                seq,num = lw_seq(i)
                if(num==1):
                        write_fasta(i,str(lw_seq(i)[0]),"lw_dna.fasta")
        
        pruned_tree.write(format=1, outfile="/Users/apple/Desktop/vis-trans/SMBE/Fig3/lw/1.raw_data/pruned_lw_tree.tre")

process()