"""
Plots quality control from busco to pruned tree to of Diurnal/ nocturnal species et. al with opsins""" 
import ete3
import numpy as np
from ete3 import Tree, TreeStyle, faces, TextFace, CircleFace, NodeStyle
import pandas as pd
import os
import sys
import re
count_col = 0
gdf = None
gt = None
qc_df = None
dict_n = None
crit = 1 #criteria for filtering species , 1 = atleast 1 opsins
#Ensure paths for input processes are correct, either copy files to location of script or change path
def input_tree():
        t=Tree("Orig_Fig.S1.tre_1kite-paper.tre")
        #t = Tree("((H:0.3,I:0.1):0.5, A:1, (B:0.4,(C:1,(J:1, (F:1, D:1):0.5):0.5):0.5):0.5);")
        return t

def input_data_frame():
    #Inputs a data frame needed for analysis
    df=pd.read_csv("diel-opsin-raw.csv")
    return df
def input_data_frame_for_qc():
    #Inputs a data frame needed for analysis

    global qc_df
    qc_df=pd.read_csv("opsin_busco.csv")

def cleanup(df):
    #cleans spaces and makes all column names lowercase
    df=df.rename(columns=lambda x: x.lower().replace(" ", "_"))
    return df
def get_leaf_names(t):
        #gets the names of all the leaves and returns a numpy array of the names
        leaf_labels=np.array([],dtype='object')
        for leaf in t.get_tree_root():
                # Do some analysis on node
                leaf_labels=np.append(leaf_labels,leaf.name)
        return leaf_labels

def search_name_in_tree(search_string,t):
        #Input  tree and string to be searched 
        #converts the list of leaf names to a list and searches 
        list_of_names=get_leaf_names(t).tolist()
        mylist=list_of_names
        return next((s for s in list_of_names if search_string in s), None)
def data_proc(df):
        remove_uncertain=False
        filter_criteria="N and D"                     
        "takes an input dataframe and cleans based on a criteria"
        #trim to needed columns
        df=df.iloc[:,[0,1,2,3,4,15,17,20,21,22,23,24,5]]
        #Rename a specific column header to something else
        df=df.rename(columns={"library-id_/_id": "library-id"}) # takes original name
        df=df.rename(columns={df.columns[6]: "diel-habitat"}) # takes position in df.iloc
        #replace empty characters with 0 
        df=df.fillna(0)
        #dh=Column that contains all the diel habitat states
        if remove_uncertain:
            dh=df["diel-habitat"].astype(str)
        elif not remove_uncertain:
            #Keeps uncertain values of diel states and removes the ? symbol
            df=df.replace({"\?":""},regex=True)    
            dh=df["diel-habitat"].astype(str)
        #Replace ? with empty characters or 0 depending on stte
        df=df.replace({"\?":""},regex=True)    
        #replace cleaned columns as float to allow for further processing
        df.rh5_blue=df.rh5_blue.astype(float)                                                                                                  
        df.rh6_lw=df.rh6_lw.astype(float)
        # Adds column for number of opsins
        df.loc[(df.rh4_uv >= 1) | (df.rh5_blue >= 1) | (df.rh6_lw >=1) | (df.rh7 >= 1), "number_opsins" ] = (df.rh4_uv+df.rh5_blue+df.rh6_lw+df.rh7)
        # Adds column for number of duplications
        df.loc[(df.rh4_uv >= 2) | (df.rh5_blue >= 2) | (df.rh6_lw >=2) | (df.rh7 >= 2), "duplications" ] = 1 
        df.loc[df.number_opsins >=1,"truename"] = ""
        df.loc[df.number_opsins >=1,"busco-score"] = ""
        return df

def filter_data(df):
        #gapminder.assign(pop_in_millions=gapminder['pop']/1e06).head(3) 
        names_list=np.array([""],dtype=object)
        #1. More than 1 opsin
        dh=list(df["diel-habitat"].astype(str))
        t=input_tree()
        global crit
        if(crit==1):
                for i,val in enumerate(dh):
                        if(df.number_opsins.iloc[i]>=1):
                                tmp_name=df.genus.iloc[i]+"_"+df.species.iloc[i]
                                new_name=search_name_in_tree(tmp_name,t) # searches for genus_species name in tree
                                df.iloc[i,df.columns.get_loc("truename")]=new_name 
                                names_list=np.append(names_list,tmp_name)

        return names_list,df
def uv_num(name):
        #returns number of uv opsins
        global gdf
        true_names=gdf["truename"].astype(str)
        for i in range(true_names.size):
                if(true_names[i]==name):
                        ret_val=gdf.rh4_uv.iloc[i]
                        break
                else:
                        ret_val=0.0
        return ret_val
def blue_num(name):
        #returns number of blue opsins
        global gdf
        true_names=gdf["truename"].astype(str)
        for i in range(true_names.size):
                if(true_names[i]==name):
                        ret_val=gdf.rh5_blue.iloc[i]
                        break
                else:
                        ret_val=0.0
        return ret_val
def lw_num(name):
        #returns number of lw opsins
        global gdf
        true_names=gdf["truename"].astype(str)
        for i in range(true_names.size):
                if(true_names[i]==name):
                        ret_val=gdf.rh6_lw.iloc[i]
                        break
                else:
                        ret_val=0.0
        return ret_val    
def rh7_num(name):
        #returns number of rh7 opsins
        global gdf
        true_names=gdf["truename"].astype(str)
        for i in range(true_names.size):
                if(true_names[i]==name):
                        ret_val=gdf.rh7.iloc[i]
                        break
                else:
                        ret_val=0.0
        return ret_val         
def trunc_nam(name):
        #Returns the genus and species
        global gdf
        true_names=gdf["truename"].astype(str)
        for i in range(true_names.size):
                if(true_names[i]==name):
                        #ret_val= gdf.superfamily.iloc[i]+" "+gdf.family.iloc[i]+" " + gdf.genus.iloc[i]+" "+gdf.species.iloc[i]
                        ret_val= gdf.genus.iloc[i]+" "+gdf.species.iloc[i]+" "+gdf.superfamily.iloc[i][:4]
                        
                        break
                else:
                        ret_val=0.0
        return ret_val
def diel_habit(name):
        #Returns the genus and species
        global gdf
        true_names=gdf["truename"].astype(str)
        for i in range(true_names.size):
                if(true_names[i]==name):
                        ret_val= gdf["diel-habitat"].iloc[i]
                        break
                else:
                        ret_val=0.0
        return ret_val
def busco_nam(name):
        #Returns the genus and species
        global gdf
        global qc_df
        true_names=gdf["truename"].astype(str)
        for i in range(true_names.size):
                if(true_names[i]==name):
                        global dict_n
                        tmp=dict_n.get(true_names[i])
                        if(tmp is not None):
                            ret_val= qc_df[qc_df['trans_name']==tmp]['c-score'].item()
                        break
                else:
                        ret_val=0.0
        return ret_val

def split_returner(s):
    "Takes input string and returns genus,species or identifier"
    unit_test=['final_soap_SRR3721684.fa','final_soap_Xter2.fa','Macrosiphum_euphorbiae.fa']
    #unit_test=["LEP_SRR2924893_22585_ORF1_Rh6_2",
    #"LEP_Myc_nea_C169037_a_79_0_l_1083_ORF1_Rh6",
    #"LEP_Ptys_c104231_g1_i1_ORF1_Rh6"]

    x=re.compile('final_soap_([A-Z0-9]+?).fa').match(s)# Finds SRR29234 like things
    if(x is not None):
        return x.group(1)
    x=re.compile('CLEAN_([a-zA-Z0-9]+?).fa').match(s) # Finds matches for genus_species all letters
    if(x is not None):
        output= x.group(1)
        return output
    x=re.compile('Trinity_([a-zA-Z0-9]+?).fa').match(s) # Finds matches for genus_species all letters
    if(x is not None):
        output= x.group(1)
        return output
    x=re.compile('([a-zA-Z]{3})_([a-zA-Z0-9]+?)\.fa').match(s) # Finds and returns Ptys_
    if(x is not None):
        output= x.group(1)+"_"+x.group(2)
        return output
    x=re.compile('([a-zA-Z0-9]+?-[a-zA-Z0-9]+?)\.tsa\.fa').match(s) # Finds and returns Library ID
    if(x is not None):
        return x.group(1)

def get_dict_names(ver):
    "Return dictionoary with keys as true names and assembly names as if flag verbose = true print other values"
    global qc_df
    ls1=list(qc_df["trans_name"])
    check=list(map(split_returner,ls1))
    dict_true_name={}
    #To turn on troubleshooting uncomment print statements below
    for i,name in enumerate(check):
        if(name is None):
            if(ver): #no sp_identifier in regex split
                print(ls1[i],"->",name,"->")
        elif(type(find_in_df(name)) is float):
            if(ver): # Regex split works and found in database, but no opsins checks for Nan type float
                print(ls1[i],"->",name,"->","found but no opsins")
        elif(find_in_df(name) is None):
            if(ver):
                # Regex split works and not found in database 
                print(ls1[i],"->",name,"->","not found in database names")    
        else:
            dict_true_name[find_in_df(name)]= ls1[i]
            #print(ls1[i],"->",name,"->",find_in_df(name))
    #unit test cases
    return dict_true_name  

def find_in_df(search_string):
    "Searches dataframe for partial match and returns index number"
    global gdf
    name_index=gdf.columns.get_loc("truename")
    assembly_index=gdf.columns.get_loc("species")

    col=["truename","genus","species","assembly","library-id"]
    for i in col:
        x=gdf[gdf[i].str.contains(search_string,na=False)]
        if(not x.empty):
                true_name=x.iloc[0,name_index]
                return true_name

def find_in_df_row_index(search_string):
    """Searches dataframe for partial match and returns row index numbers"""
    global gdf
    col=["truename","genus","species","assembly","library-id"]
    for i in col:
        x=gdf[i].str.contains(search_string,na=False)
        for j,bval in enumerate(x):
            if (bval):
                return j
def get_scores(ver):
    global gdf
    "Return index numbers from gf as input key and busco score as output key. with keys as true names and assembly names as if flag verbose = true print other values"
    global qc_df
    ls1=list(qc_df["trans_name"]) # list of names of transcriptomes
    pvi=qc_df.columns.get_loc('c-score') # column index of busco
    check=list(map(split_returner,ls1))
    #To turn on troubleshooting put ver = true
    for i,name in enumerate(check):
        score=qc_df.iloc[i,pvi]
        if(name is None):
            if(ver): #no sp_identifier in regex split
                print(ls1[i],"->",name,"->","has no sp identifier in regex split")
        elif(find_in_df(name) is None):
            if(ver):
                # Regex split works and not found in database 
                print(ls1[i],"->",name,"->","not found in database names")    
        else:
            if(not ver):
                #df.loc[df.P == 1002.8, 'T'].values[0]
                #dict_true_name[find_in_df(name)]= ls1[i]
                gdf_index=find_in_df_row_index(name)
                print(ls1[i],">",name,">",gdf.iloc[gdf_index,5],">",gdf.iloc[gdf_index,3],"_",gdf.iloc[gdf_index,4])# troubleshooting
                #print(gdf.iloc[gdf_index,3],",",gdf.iloc[gdf_index,4],",",ls1[i],",",name,",",gdf.iloc[gdf_index,5],",",gdf.iloc[gdf_index,14])
                #Add scores to gdf
                gdf.iloc[gdf_index,gdf.columns.get_loc("busco-score")]=qc_df.iloc[i,1]  #col. index 2 = percent visual score, 
  

def process():
    t=input_tree()
    df=input_data_frame()
    df=cleanup(df) # has set copy warnings
    df=data_proc(df)
    input_data_frame_for_qc() # inputs quality control dataset
    global qc_df #refers to the global qc df
    filtered_names,df=filter_data(df)
    #true_names=[""]
    #for s in filtered_names:
    #        true_names.append(search_name_in_tree(s,t))
    #pruned_tree=input_tree()
    #true_names=list(filter(None,true_names))
    true_names=list(df.truename.dropna()) #one line true names
    #global gt 
    #gt = t
    global gdf 
    gdf = df
    get_scores(False)
    gdf.to_csv("busco-score_raw.csv")
    #test cases

    global dict_n
    #dict_n=get_dict_names(False)
    #pruned_tree.prune(true_names)
    #pruned_tree.ladderize()
    #runed_tree.convert_to_ultrametric()
    #add_faces(pruned_tree)
    #pruned_tree.show()
    #pruned_tree.write(format=1, outfile="pruned_tree.tre")

def add_faces(t):
        # Basic tree style
        ts = TreeStyle()
        ts.force_topology= False
        ts.mode = "c"
        ts.scale= None
        ts.optimal_scale_level="mid"
        ts.layout_fn=layout
        ts.show_leaf_name = False
        ts.arc_start = -90 # 0 degrees = 3 o'clock
        ts.arc_span = 180
        #t.show(tree_style=ts)
        t.render("Fig1b_busco_extra_names.svg",tree_style=ts)
        
def layout(node):
        if node.is_leaf():
                leaf_name=node.name     
                dh=diel_habit(leaf_name)
                #if(dh=="D"or dh=="B" or dh =="C"):
                #        node.set_style(NodeStyle())
                #        node.img_style["bgcolor"] = "Yellow"
                #elif(dh=="N"):
                #        node.set_style(NodeStyle())
                #        node.img_style["bgcolor"] = "Silver"

                # changes genus and species color based in diel activity
                
                uv_size=int(uv_num(leaf_name))
                blue_size=int(blue_num(leaf_name))
                lw_size=int(lw_num(leaf_name))*1
                rh7_size=int(rh7_num(leaf_name))*1
                genus_species=trunc_nam(leaf_name)
                busco=busco_nam(leaf_name) 
                if(genus_species!=0.0):
                        genus_species=genus_species[:40]   
                        if(len(genus_species)<40):
                                num_space=(40-len(genus_species))*" "
                                genus_species=genus_species+num_space
                gs=TextFace(genus_species,fstyle="italic")
                shift = 1
                draw_aligned_faces_as_table=True

                if(uv_size >1 or blue_size >1 or lw_size >1):
                        node.img_style["size"]=5
                        node.img_style["fgcolor"]="red"

                if(dh=="D"):
                        gs.background.color = "gold"
                elif(dh=="B" or dh =="C"):
                        gs.background.color = "Green"
                elif(dh=="N"):
                        gs.background.color = "aqua"
                faces.add_face_to_node(gs,node, column=0)
                #uv
                uv_face= TextFace("  " +str(uv_size)+"  ")
                uv_face.background.color = "Plum"
                uv_face.border.width = 1
                if(uv_size>1):
                        uv_face.opacity = 1*0.5*uv_size
                        uv_face.background.color = "Orchid"
                elif(uv_size==0):
                        uv_face.opacity = 0
                        uv_face.background.color = "White"
                faces.add_face_to_node(uv_face, node, column=2, aligned=True,position = "branch-right")
                
                #blue
                blue_face =TextFace("   " + str(blue_size)+ "  ",fgcolor="black")
                blue_face.background.color ="LightSkyBlue"
                blue_face.border.width=1
                
                if(blue_size>1):
                        blue_face.opacity = 1*0.5*blue_size
                        blue_face.background.color ="SteelBlue"
                elif(blue_size==0):
                        blue_face.opacity = 0
                        blue_face.background.color ="White"
                faces.add_face_to_node(blue_face, node, column=3,aligned=True)
                #lw
                lw_face=TextFace("   "+ str(lw_size)+"   ")
                lw_face.background.color ="PaleGreen"
                lw_face.border.width=1
                if(lw_size>1):
                        lw_face.opacity = 1*0.5*lw_size
                        lw_face.background.color ="LimeGreen"
                elif(lw_size==0):
                        lw_face.opacity = 0
                        lw_face.background.color ="White"
                faces.add_face_to_node(lw_face, node, column=4,aligned=True)
                #rh7
                rh7_face=TextFace("   "+ str(rh7_size)+"   ")
                rh7_face.background.color ="LightCoral"
                rh7_face.border.width=1
                if(rh7_size>1):
                        rh7_face.opacity = 1*0.5*rh7_size
                        rh7_face.background.color ="Crimson"
                elif(rh7_size==0):
                        rh7_face.opacity = 0
                        rh7_face.background.color ="White"
                faces.add_face_to_node(rh7_face, node, column=5,aligned=True)
                
                #Add Busco scores
                busco_face=TextFace("   "+ str(busco)+"   ")
                busco_face.background.color ="White"
                busco_face.border.width=1
                faces.add_face_to_node(busco_face, node, column=6,aligned=True)
                
                node.convert_to_ultrametric()
                # #amount of columns added after the genus and species name face
                # faces.add_face_to_node(gs,node, column=0)
                # for i in range(uv_size):
                #         faces.add_face_to_node(CircleFace(radius=6,color="Violet",style="circle"), node, column=shift+i, position = "aligned")
                # for i in range(blue_size):
                #         faces.add_face_to_node(CircleFace(radius=6,color="DarkBlue",style="circle"), node, column=shift+2+i, position = "aligned")
                # for i in range(lw_size):
                #         faces.add_face_to_node(CircleFace(radius=6,color="Green",style="circle"), node, column=shift+6+i, position = "aligned")
                # for i in range(rh7_size):
                #         faces.add_face_to_node(CircleFace(radius=6,color="Red",style="circle"), node, column=shift+10+i, position = "aligned")

                #black is placeholder to be removed in illustrator
                #global count_col
                #if(count_col==0):      
                #        faces.add_face_to_node(CircleFace(radius=6,color="Black",style="circle"),node, column =shift+11+0,position = "aligned")
                #        count_col+=1
                
                faces.add_face_to_node(TextFace("     "),node, column=1) #adds spaces to the names to increase length
        else:
                node.img_style["fgcolor"] = "black"
                node.img_style["size"] = 0


if __name__ == "__main__":
    process()