"""Analysis of opsin and diel activity using a dataset of species which we are sure about the diel activity Table S1
Convert transcriptome annotation table s1 to a csv file named diel-opsin-raw.csv before running. 
 Set 0 Run all possible combinations of sets and true false
    Set 1 Both Crepuscular/ both or diurnal are considered as Diurnal in the analysis Grouping is Nocturnal vs. everything else
    Set 2 Both Crepuscular and both are considered nocturnal, grouping is Diurnal vs. everyhing else
    Set 3 Only strictly nocturnal and diurnal species
    ignore duplicates can be set to true or false depending on whether uncertain groups are to be removed or not"""
    
import numpy as np
import matplotlib.pyplot as plt
import scipy.stats as st
import math
import pandas as pd
import sys
import os
## https://www.shanelynn.ie/using-pandas-dataframe-creating-editing-viewing-data-in-python/ for help
def input_data():
    #Inputs a data frame needed for analysis
    path=os.getcwd()
    df=pd.read_csv(path+"/diel-opsin-raw.csv")
    return df

def cleanup(df):
    #cleans spaces and makes all column names lowercase
    df=df.rename(columns=lambda x: x.lower().replace(" ", "_"))
    return df
def diel_duplication_trim(df,remove_uncertain,set):
    """Specific processing to create array for processing how duplications are associated with diel niche. Uses the chi- square test. 
    This functions counts the number of duplications and number of no duplications for day and night flying species
    by specifiying remove_uncertain, dataset can be modified"""
    #trim to needed columns
    df=df.iloc[:,[0,1,2,3,4,15,17,20,21,22,23,24]]
    #Rename a specific column to something else
    df=df.rename(columns={df.columns[6]: "diel-habitat"})
    #replace empty characters with 0 
    df=df.fillna(0)
    #dh=Column that contains all the diel habitat states
    if remove_uncertain:
        dh=df["diel-habitat"].astype(str)
    elif not remove_uncertain:
        #Keeps uncertain values of diel states and removes the ? symbol
        df=df.replace({"\?":""},regex=True)    
        dh=df["diel-habitat"].astype(str)
    #Replace ? with empty characters or 0 depending on stte
    df=df.replace({"\?":""},regex=True)    
    #replace cleaned columns as float
    df.rh5_blue=df.rh5_blue.astype(float)                                                                                                  
    df.rh6_lw=df.rh6_lw.astype(float)
    # Adds column for number of opsins
    df.loc[(df.rh4_uv >= 1) | (df.rh5_blue >= 1) | (df.rh6_lw >=1) | (df.rh7 >= 1), "number_opsins" ] = (df.rh4_uv+df.rh5_blue+df.rh6_lw+df.rh7)
    # Adds column for number of duplications
    df.loc[(df.rh4_uv >= 2) | (df.rh5_blue >= 2) | (df.rh6_lw >=2) | (df.rh7 >= 2), "duplications" ] = 1 

    dd=df["duplications"]
    dup_N=0
    ndup_N=0
    dup_D=0
    ndup_D=0
    if set==1:
        for i in range(dh.size):
            if(df.number_opsins.iloc[i]>=1):    
                    if(dh[i]== "N"):
                        if(dd[i]==1): 
                            dup_N+=1
                        else:
                            ndup_N+=1
                    elif(dh[i]=="D" or dh[i] == "C" or dh[i] == "B"):
                        if(dd[i]==1): 
                            dup_D+=1
                        else:
                            ndup_D+=1
    elif set==2:
        for i in range(dh.size):
                if(df.number_opsins.iloc[i]>=1):
                    if(dh[i]== "N"or dh[i] == "C" or dh[i] == "B"):
                        if(dd[i]==1): 
                            dup_N+=1
                        else:
                            ndup_N+=1
                    elif(dh[i]=="D"):
                        if(dd[i]==1): 
                            dup_D+=1
                        else:
                            ndup_D+=1
    elif set ==3:
        for i in range(dh.size):
                if(df.number_opsins.iloc[i]>=1):
                    if(dh[i]== "N"):
                        if(dd[i]==1): 
                            dup_N+=1
                        else:
                            ndup_N+=1
                    elif(dh[i]=="D"):
                        if(dd[i]==1): 
                            dup_D+=1
                        else:
                            ndup_D+=1
    output(dup_N,ndup_N,dup_D,ndup_D,remove_uncertain,set)
def output(dup_N,ndup_N,dup_D,ndup_D,remove_uncertain,set):
    dict_titles={
        1: "N. vs D+C+B",
        2: "D. vs. N+C+B",
        3: "N vs. D"
        }
    #Set 1 Both Crepuscular/ both or diurnal are considered as Diurnal in the analysis Grouping is Nocturnal vs. everything else
    # Set 2 Both Crepuscular and both are considered nocturnal, groupinf is Diurnal vs. everyhing else
    
    obs=[[dup_N,ndup_N],[dup_D,ndup_D]]
    chi2, p, dof, expctd = st.chi2_contingency(obs)
    print(dict_titles[set])
    print("Remove uncertain values:",remove_uncertain)
    print("observed frequencies \n","N:",obs[0],"\n","D:",obs[1])
    print("chi2 = ",chi2)
    print("p-value = ",p)
    print("degrees of freedom =",dof)
    
def str2bool(v):
  return v.lower() in ("yes", "true", "t", "1")
def main():
    #Needs input of dataset and whether to remove uncertain
    df=input_data()
    df=cleanup(df)
    input1 = sys.argv[1]
    input2 = sys.argv[2]
    #Set 0 Run all possible combinations of sets and true false
    #Set 1 Both Crepuscular/ both or diurnal are considered as Diurnal in the analysis Grouping is Nocturnal vs. everything else
    #Set 2 Both Crepuscular and both are considered nocturnal, grouping is Diurnal vs. everyhing else
    #Set 3 Only strictly nocturnal and diurnal species
    #ignore duplicates can be set to true or false depending on whether uncertain groups are to be removed or not
    set = int(input1)
    if(set==0):
        for i in np.arange(1,4,1):
            diel_duplication_trim(df,False,i)
            diel_duplication_trim(df,True,i)
    else:
        remove_uncert=str2bool(input2)
        diel_duplication_trim(df,remove_uncert,set)
if __name__ == '__main__':
  main()